#include <stdio.h>
#include <stdlib.h>
#include "except.h"
#include "bool.h"
#include "base.h"
#include "frequencies.h"
#include "maxmin.h"
#include "permute.h"
#include "dist.h"
#include "distvector.h"

#define LETTERS 26

/*
 * ----------
 * Original:  #define MAXWIDTH 70
 * ----------
 */

#define MAXWIDTH 500
#define COMMENTS 5


static int aa_index_table[LETTERS] =
/*A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z*/
{0, -1, 1, 2, 3, 4, 5, 6, 7,-2, 8, 9,10,11,-2,12,13,14,15,16,-2,17,18,-1,19,-1};

static int
index_translation (int i) {
  return aa_index_table[i];
}


static void
read_line (FILE *fp) {
  while (fgetc(fp) != '\n') ;
}


extern int
read_pssm (int **pssm, FILE *fp) {
  int i, index, value, position = 0;

  for (i = 0; i < COMMENTS; ++i) {
    read_line(fp);
  }
  while (fscanf(fp,"%d",&value) > 0) {
    pssm[position][0] = value;
    for (i = 1; i < LETTERS; ++i) {
      if ((index = index_translation(i)) == -2) { /* don't read */
      } else if (index == -1) {	/* read but ignore */
	fscanf(fp,"%d",&value);
      } else {
	fscanf(fp,"%d",&value);
	pssm[position][index] = value;
      }
    }
    read_line(fp);
    ++position;
  }
  return position;
}

static void
parse_arguments (char **profilefile, bool *floor, bool *orderedp,
		 int *order, bool *continuous, 
		 bool *steps, int *fromsig, int *tosig,
		 double *p_threshold, int argc, char *argv[]) {
  ++argv; --argc;
  while (argc > 0) {
    if ((*argv)[0] == '-') {
      if ((*argv)[1] == 'p') {
	*p_threshold = atof(argv[1]); argc -= 2; ++argv; ++argv;
      } else if ((*argv)[1] == 'o') {
	*order = atoi(argv[1]); argc -= 2; ++argv; ++argv;
      } else if ((*argv)[1] == 'c') {
	*continuous = TRUE; argc -= 1; ++argv;
      } else if ((*argv)[1] == 'q') {
	*steps = TRUE; argc -= 1; ++argv;
      } else if ((*argv)[1] == 'F') {
	*floor = TRUE; argc -= 1; ++argv;
      } else if ((*argv)[1] == 'P') {
	*orderedp = TRUE; argc -= 1; ++argv;
      } else if ((*argv)[1] == 'f') {
	*fromsig = atoi(argv[1]); argc -= 2; ++argv; ++argv;
      } else if ((*argv)[1] == 't') {
	*tosig = atoi(argv[1]); argc -= 2; ++argv; ++argv;
      } else {
	printf("I don't understand.\n");
	exit(1);
      }
    } else {
      *profilefile = *argv; argc -= 1; argv += 1;
    }
  }
}


int
main (int argc, char *argv[]) {
  Frequencies_T frequencies;
  char *profilefile = NULL;
  int **scoremat, width, *min_scores, *max_scores, order = 1, pos;
  int *ordering;
  int fromsig = 40, tosig = 1;
  double p_threshold = -1;
  FILE *fp;
  bool continuous = FALSE, steps = FALSE, floor = FALSE, orderedp = FALSE;
  Distvector_T old, new;
  Dist_T dist;

  TRY
  parse_arguments(&profilefile,&floor,&orderedp,&order,&continuous,&steps,
		  &fromsig,&tosig,&p_threshold,argc,argv);
  frequencies = Frequencies_fill(order);
  scoremat = (int **) calloc(MAXWIDTH,sizeof(int *));
  for (pos = 0; pos < MAXWIDTH; ++pos) {
    scoremat[pos] = (int *) calloc(NAT_AAS,sizeof(int));
  }
  fp = fopen(profilefile,"r");
  width = read_pssm(scoremat,fp);
  fclose(fp);

  min_scores = (int *) calloc(width,sizeof(int));
  max_scores = (int *) calloc(width,sizeof(int));
  find_bounds(min_scores,max_scores,scoremat,width);

  ordering = Permute_find_ordering(scoremat,width,frequencies);
  if (orderedp) {
    Permute_print_ordering(ordering,width);
    print_floor_ordered(scoremat,ordering,width);
  } else if (floor) {
    print_floor(scoremat,width);
  }
  old = Distvector_init(order);
  for (pos = 0; pos < width; ++pos) {
    new = Distvector_update(old,pos,order,frequencies,scoremat,
			    min_scores,max_scores);
    /* printf("%d %f\n",pos,Distvector_check(new)); */
    Distvector_free(&old);
    old = new;
  }
  dist = Distvector_compress(new);
  if (steps) {
    Dist_print_steps(dist,fromsig,tosig);
  } else if (p_threshold > 0.0) {
    Dist_print_threshold(dist,p_threshold,continuous);
  } else {
    Dist_print_probs(dist,continuous);
  }
  ELSE
    fprintf(stderr,
	    "An internal error has occured from which there is "
	    "no recovery.\nPlease report this error by e-mail to "
	    "thomas.wu@stanford.edu:\n\n");
  RERAISE;
  END_TRY;
  return EXIT_SUCCESS; 
}
